/*
 * Copyright (c) 2010, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using System.Text.RegularExpressions;
using System.IO;
using System.Collections;

namespace MobiServer.Devices.DMX
{

    public partial class ControlDevice : MobiServer.ControlBase
    {
        #region Constants/Variables
 
        public EventHandlerChannel dmxWriteHandler;
        public MobiServer.ServerEventHandler serverWriteHandler;

        List<ControlChannel> channelList = new List<ControlChannel>();
        ControlChannel targetControl = null;
        int targetHeight = 0;

        string deviceName = "";
        int channelBase = 0;
        int channelMax = 0;

        #endregion

        #region Constructor/Destructor Attach/Dettach
        
        public ControlDevice()
        {
            InitializeComponent();
        }

        public ControlDevice(MobiServer.MessageControl messageControl, string deviceName, int channelBase, int channelMax)
            : base(messageControl)
        {
            InitializeComponent();
            this.deviceName = deviceName;
            this.channelBase = channelBase;
            this.channelMax = channelMax;
            InitChannels(messageControl);
            LoadSettingDevice();
        }

        private void ControlDevice_Load(object sender, EventArgs e)
        {
            this.Text = (String.Format("{0}: {1}`", deviceName, channelBase));
            for(int i= 0; i< channelList.Count;i++)
            {
                ControlChannel channelControl = (ControlChannel)channelList[i];
                channelControl.Text = (String.Format("{0}({1})", i + 1, channelBase + i));
            }
            return;
        }

        void InitChannels(MobiServer.MessageControl messageControl)
        {
            ControlChannel channelControl = null;
            for (int i = 0; i < channelMax; i++)
            {
                channelControl = new ControlChannel(messageControl, this, i + 1, 0);
                
                channelList.Add(channelControl);
                panelChannel.Controls.Add(channelControl);
                channelControl.Left = ((channelControl.Width + 5) * i);
            }

            panelChannel.Width = channelMax * (channelControl.Width + 5);
            groupBoxBase.Width = panelChannel.Width + 10;
            this.Width = groupBoxBase.Width + 10;
            buttonDettach.Left = groupBoxBase.Width - 15;
            buttonUpdateAll.Left = groupBoxBase.Width - 15;

            panelChannel.Height = channelControl.Height + 10;
            groupBoxBase.Height = panelChannel.Height + 20;
            this.Height = groupBoxBase.Height + 10;
        }

        private void buttonDettach_Click(object sender, EventArgs e)
        {
            Dettach(this.groupBoxBase.Enabled);
            if (groupBoxBase.Enabled)
            {
                CreateServerEvent(DataDevice.States.Attach);
            }
            else
            {
                CreateServerEvent(DataDevice.States.Dettach);
            }
        }

        public void Dettach(bool flag)
        {
            if (flag)
            {
                buttonDettach.Image = MobiServer.Devices.DMX.Properties.Resources.attach;
                this.groupBoxBase.Enabled = false;
                this.SaveSettingDevice();
            }
            else
            {
                this.groupBoxBase.Enabled = true;
                buttonDettach.Image = MobiServer.Devices.DMX.Properties.Resources.close;
            }
        }

        public string DeviceName {
            get { return this.deviceName; }
        }

        public int ChannelBase
        {
            get { return this.channelBase; }
        }

        public int ChannelMax
        {
            get { return this.channelMax; }
        }

        #endregion

        #region Events to DMX Hardware.

        internal void SendCommand(int channelCnt, int data)
        {
            if (channelCnt < 0 || channelCnt >= channelMax)
            {
                return;
            }

            if (data > 255)
            {
                data = 255;
            }
            else if (data < 0)
            {
                data = 0;
            }

            int channel = (channelBase - 1) + channelCnt;

            EventArgsChannel e = new EventArgsChannel(channel, (byte)data);
            if (dmxWriteHandler != null)
            {
                dmxWriteHandler(this, e);
            }
        }

        private void buttonUpdateAll_Click(object sender, EventArgs e)
        {
            for (int ch = 0; ch < channelMax; ch++)
            {
                SendCommand(ch, channelList[ch].Value);
            }
        }

        //Future Function: Read DMX In Events.
        /*
        public void OnSerialReadEvent(object sender, DMX.EventArgs e)
        {
            //if (Contains(e.Channel))
            //{
            //}

        }*/

        #endregion

        #region Server In/Out

        public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckAddress(e.Args))
                {

                    DataDevice d = DataDevice.Parse(e.Args);
                    if (d.Type == DataDevice.TYPE_DIRECT)
                    {
                        d.ChannelNum = d.ChannelNum - channelBase;
                    }

                    if (d != null)
                    {
                        if (d.State == DataDevice.States.Dettach)
                        {
                            Dettach(true);
                        }
                        else if (d.State == DataDevice.States.Out)
                        {                
                            ((ControlChannel)channelList[d.ChannelNum]).Value = d.Value;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }


        void CreateServerWriteEvent(string[] args)
        {
            MobiServer.ServerEventArgs e = new MobiServer.ServerEventArgs(args);
            if (serverWriteHandler != null)
            {
                serverWriteHandler(this, e);
            }
        }


        void CreateServerEvent(string state)
        {
            switch (state)
            {
                case DataDevice.States.Attach:
                case DataDevice.States.Dettach:
                    DataDevice d = new DataDevice(state, deviceName, channelBase, channelMax);
                    string[] args = d.ToArray();
                    this.CreateServerWriteEvent(args);
                    break;
            }
        }


        #endregion

        #region Save / Load

        string GetFilenameDevice()
        {
            string str = Application.StartupPath + @"\" + ControlDevice.SETTING_DIRECTORY + @"\" + String.Format("DMX_{0}_{1}_{2}.txt", channelBase, channelMax, DeviceName);
            return str;  
        }
   
        public static Hashtable LoadDeviceAll(MessageControl messageControl)
        {
            Hashtable holder = new Hashtable();

            string[] files = Directory.GetFiles(Application.StartupPath + @"\" + ControlDevice.SETTING_DIRECTORY);

            foreach (string file in files)
            {
                string f = Path.GetFileNameWithoutExtension(file);
                Match m = Regex.Match(f, @"^DMX_(\d+)_(\d+)_(.+)");
                if (m.Success)
                {
                    int channelBase = int.Parse(m.Groups[1].ToString());
                    int channelMax = int.Parse(m.Groups[2].ToString());
                    string deviceName = m.Groups[3].ToString();
                    ControlDevice control = new ControlDevice(messageControl, deviceName, channelBase, channelMax);

                    if (control.GetEnabled())
                        holder.Add(holder.Count, control);
                }
            }

            return holder;
        }

        public void SaveSettingDevice()
        {
            StreamWriter fileStream = null;
            string file = GetFilenameDevice();
            try
            {
                fileStream = new StreamWriter(file, false);

                fileStream.WriteLine("{0}", this.GetEnabled());
                
                for (int i = 0; i < channelList.Count; i++)
                {
                    ControlChannel c = (ControlChannel)channelList[i];
                    fileStream.Write("{0}\t", c.Value);
                }
                fileStream.WriteLine();
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public void LoadSettingDevice()
        {
            StreamReader fileStream = null;
            string file = GetFilenameDevice();
            string line = "";
            string[] args = null;

            try
            {
                if (!File.Exists(file))
                    return;

                fileStream = new StreamReader(file);

                line = fileStream.ReadLine();
                this.groupBoxBase.Enabled = bool.Parse(line);

                line = fileStream.ReadLine();
                args = line.Split("\t".ToCharArray());
                for (int i = 0; i < args.Length; i++)
                {
                    if (args[i] == "")
                        continue;

                    byte data = byte.Parse(args[i]);
                    ((ControlChannel)channelList[i]).Value = data;
                }

            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion


        #region Check address and others
      
        public bool CheckAddress(string deviceName, int channel)
        {
            if (deviceName == DataDevice.TYPE_DIRECT)
            {
                if (channel >= channelBase && channel < (channelBase + channelMax))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else if (deviceName == this.deviceName)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        bool CheckAddress(string[] args)
        {
            string deviceName = args[2];
            int channel = int.Parse(args[3]);
            bool flag = CheckAddress(deviceName, channel);
            return flag;
        }

        public bool CheckDeviceNameExist(string newName)
        {

            if (this.deviceName == newName)
            {
                return true;
            }
            else
            {
                return false;
            }
                
        }

        public bool CheckDeviceChannelExist(int newBase, int newMax)
        {
            if (newBase >= (channelBase + channelMax) ||
                     (newBase + newMax) < channelBase)
            {
                return false;
            }
            else
            {
                return true;
            }

        }
        
        public bool GetEnabled()
        {
            return this.groupBoxBase.Enabled;
        }

        #endregion



    }
}
