﻿/*
 * Copyright (c) 2010, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace MobiServer.Devices.DMX
{
    
    public class DataDevice : MobiServer.DataBase
    {
        public const string HeaderDMX = "DMX";
        public const string TYPE_DIRECT = "Direct";
        int channelBase = 0;
        int channelMax = 0;

        int channelNum = 0;
        byte value = 0;

        public DataDevice(string state, string deviceName, int data1, int data2)
            : base(HeaderDMX)
        {
            //Header = HeaderDMX;
            this.State = state;
            this.Type = deviceName;
            
            switch (this.State)
            {
                case States.Attach:
                case States.Dettach:
                    this.channelBase = data1;
                    this.channelMax = data2;
                    break;
                case States.In:
                case States.Out:
                    this.channelNum = data1;
                    this.value = (byte)data2; 
                    break;
            }
        }

        public int ChannelBase
        {
            get { return this.channelBase; }
        }

        public int ChannelMax
        {
            get { return this.channelMax; }
        }

        public int ChannelNum
        {
            get { return this.channelNum; }
            set { this.channelNum = value; }
        }

        public byte Value
        {
            get { return this.value; }
            set { this.value = value; }
        }

        public override string[] ToArray()
        {
            string[] args = null;

            switch (State)
            {
                case States.Attach:
                case States.Dettach:
                    args = new string[5] { Header, State, Type, ChannelBase.ToString(), ChannelMax.ToString()};
                    break;
                case States.In:
                case States.Out:
                    args = new string[5] { Header, State, Type, ChannelNum.ToString(), Value.ToString() };
                    break;
            }
            return args;
        }

        static public DataDevice Parse(string[] args)
        {
            try
            {
                DataDevice d = null;

                if (args[0] != DataDevice.Header)
                    return null;

                string state = args[1];
                string deviceName = args[2];

                switch (state)
                {
                    case States.Attach:
                    case States.Dettach:
                        int channelBase = int.Parse(args[3]);
                        int channelMax = int.Parse(args[4]);
                        d = new DataDevice(state, deviceName, channelBase, channelMax);
                        break;
                    case States.In:
                    case States.Out:
                        int channelNum = int.Parse(args[3]);
                        byte value = byte.Parse(args[4]);
                        d = new DataDevice(state, deviceName, channelNum, (int)value);
                        break;
                }
                
                return d;
            }
            catch (Exception ex)
            {
                return null;
            }

        }
    }
}

